#!/usr/bin/env python3

## appels :
# dico= lectureFichier(PATH+fname1)
# ecritureCSV(dico, PATH+fname2)
##

# chemin qui mène à votre répertoire de travail
PATH = "/DONNEES/Enseignement/2018-19/SNT/TPs-donnees/"
# nom du fichier xml téléchargé depuis www.data.gouv.fr
#fname1 = "fr-2019-e2-2019-2019-05-08-18-v.csv"
fname1 = "2019-Donnees-PollutionA.xml"
fname = "2019-Donnees-PollutionA.csv"

dicoP = {'1': "SO2", '5': 'PM10' , '7': 'O3', '8': 'NO2' , '9': 'NOX', '10': 'CO', '20': 'C6H6', '6001': 'PM2.5'}

keys = ['date', 'heure', '"code station"', 'polluant', 'NbMesures', 'Unite', 'Valeurs', 'ValsMoyenne']

def gmlTimeInstant(ligne): #  <gml:TimeInstant…36_FR36032_6001_4_2019-05-08-184632">
    stat, pollutant = ligne.split('_')[-4:-2]
    dateHeure = ligne.split('_')[-1] # 2019-05-08-184632">
    ldh = dateHeure.split('"')[0]  # 2019-05-08-184632
    ldh = ldh.split('-')    # 2019, 05, 08, 184632
    date= ldh[0]+ldh[1]+ldh[2]
    heure=ldh[-1]
    return stat, pollutant, date, heure

def omObservation(ligne): # <om:OM_Observation gml:id="FR_OBS_erbjg36hd0_FR071A_FR36021_5_6">
    ligne = ligne.split('"')[1] # FR_OBS_erbjg36hd0_FR071A_FR36021_5_6
    station, pollutant = ligne.split('_')[-3:-1] # FR36021, 5
    return station, pollutant

def omObservedPollutant(ligne): # <om:observedProperty xlink…/pollutant/8"/>
    typeVal = ligne.split('"/>')[0] # <om:observedProperty xlink…/pollutant/8
    type, val = typeVal.split('/')[-2:] # pollutant, 8
    return type, val

def sweCount(ligne): # <swe:value>5</swe:value>
    c = ligne.split("</")[0] # <swe:value>5
    return c.split(">")[-1]

def sweQuantity(ligne):# <swe:uom xlink:…vocabulary/uom/concentration/ug.m-3"/>
    typeVal = ligne.split('"/>')[0] # <swe:uom xlink:…vocabulary/uom/concentration/ug.m-3
    type, val = typeVal.split('/')[-2:] # concentration, ug.m-3
    return type, val

def sweValues(ligne): # <swe:values>2019-05-07T00:00:00+00:00,2019-05-07T01:00:00+00:00,2,1,9.6@@2019-05-07T01:00:00+00:00,2019-05-07T02:00:00+00:00,2,1,9.1 … @@</swe:values>
    ligne = ligne.split("values>")[1] # 2019-05-07T00:00:00+00:00,2019-05-07T01:00:00+00:00,2,1,9.6@@2019-05-07T01:00:00+00:00,2019-05-07T02:00:00+00:00,2,1,9.1 … @@</swe:
    ligne = ligne.split("</swe")[0] # 2019-05-07T00:00:00+00:00,2019-05-07T01:00:00+00:00,2,1,9.6@@2019-05-07T01:00:00+00:00,2019-05-07T02:00:00+00:00,2,1,9.1 … @@
    lvals = ligne.split('@@')[:-1] # apres le dernier @@ il reste ''
    vals = []
    for e in lvals : # 2019-05-07T00:00:00+00:00,2019-05-07T01:00:00+00:00,2,1,9.6
        vals.append( float( e.split(',')[-1] ))
    return vals

def lectureFichier(fnom):
    file = open(fnom, "r", encoding='utf_8')
    line = file.readline()
    avanct = 0
    dicoNbP = {'1':0, '5':0, '7':0, '8':0, '9':0,'10':0,'20':0,'6001':0}
    numSection = 0
    dico={}
    llist = []
    while ( line ) : 
        """une section commence par <gml:featureMember> et termine </gml:featureMember>
        la 1ere section est inutilisee
        """
        if ( "<gml:featureM" in line ) and ( avanct==0 ) :
            avanct += 2     # 1
            """
        elif ( "<om:OM_Observation" in line ) and ( avanct==1 ) :
            avanct += 1
            s, p = omObservation(line)
            """
        elif ( "<gml:TimeInstant" in line ) and ( avanct==2 ) :
            avanct += 2     # 1
            s, p, d, h = gmlTimeInstant(line)
            dicoNbP[p] += 1
            """
        elif ( "<om:observedProperty" in line ) and ( avanct==3 ) :
            avanct += 1
            tp, p = omObservedPollutant(line)
            if v!=p : 
                print("error")
            """
        elif ( "<swe:value>" in line ) and ( avanct==4 ) :
            avanct += 1
            c = sweCount(line)
        elif ( "<swe:Quantity" in line ) and ( avanct==5 ) :
            avanct += 1
        elif ( "<swe:uom" in line ) and ( avanct==6 ) : # the uom's Quantity
            avanct += 1
            tc, vc = sweQuantity(line)
        elif ( "<swe:values>" in line ) and ( avanct==7 ) :
            vals = sweValues(line)
            if len(vals) != int(c) : 
                print(f"error : len(vals)={len(vals)} mais c={c}")
#            llist.append[{'date':d, 'heure':h, 'station':s, 'polluant':p, 'NbMesures':c, 'Unite':vc, 'Valeurs':vals}]
            dico[numSection]={keys[0]:d, keys[1]:h, keys[2]:s, keys[3]:dicoP[p], keys[4]:c, keys[5]:vc, keys[6]:vals}
            numSection +=1 
            avanct = 0
        line = file.readline()

    print("end of parser")
    file.close()
    return dico

def ecritureCSV(liste, fnom):
    from numpy import mean 
    file = open(fnom, "w", encoding='utf_8')
    
    KKeys=keys[:4]+keys[5:-1]+["Val Mesuree"] # retire le nombre de mesures
    for e in KKeys[:-2] :
        file.write(e+';')
    file.write(KKeys[-1]+"\n")
        
    for dico in liste.values() :
        for e in KKeys[:-2] :
            file.write( dico[e]+';' )
        ValM = f"{mean( dico[KKeys[-2]] ):.3f}"
        #ValM = ValM.replace('.',',') # si separateur decimal est ','    
        file.write( ValM+"\n") 
    file.close()
    print("end of writing")
    return 0

# end of file '2019-Toucan.py'
